from collections import OrderedDict
from copy import deepcopy


def _parse_command(command):
    content = []
    current = ''
    idx = 0

    while idx < len(command):
        ch = command[idx]
        if ch in ('[', '('):
            idx2, extra = _parse_command(command[idx + 1:])
            idx += idx2 + 1
            content.append({
                'op': ch == '[' and 'optional' or 'or',
                'value': extra
            })
        elif ch in (']', ')'):
            if len(current) > 0:
                content.append(current)
            return idx, content
        elif ch == '|':
            if len(current) > 0:
                content.append(current)
                current = ''
            content.append(ch)
        elif ch == ' ':
            if len(current) > 0:
                content.append(current)
                current = ''
        else:
            current += ch
        idx += 1

    if len(current) > 0:
        content.append(current)
    return idx, content


def _replace_word(part, command, args):
    for word in part.split(' '):
        if '<' in word:
            value = 'foobar'
            if word == '<interface>':
                value = 'swp1'
            elif word == '<rd>':
                value = '1:1'
            elif word == '<mac>':
                value = '00-14-22-01-23-45'
            elif word == '<bgppeer>':
                value = '1.2.3.4'
            elif word == '<ipv4/prefixlen>':
                value = '1.2.2.2/32'
            elif word == '<ipv6/prefixlen>':
                value = 'FE80:0000:0000:0000:0202:B3FF:FE1E:8329/32'
            elif word == '<ipv6>':
                value = 'FE80:0000:0000:0000:0202:B3FF:FE1E:8329'
            elif word.startswith('<ip'):
                value = '1.2.2.2'
            elif word in ('<1-16777215>', '<1-4294967295>'):
                value = '10'
            elif word == '<as-path-access-list>':
                value = 'LARRY'
            elif word in ('<community-list>', '<extcommunity-list>'):
                value = 'MARGE'
            elif word == '<prefix-list-v4>':
                value = 'BOB'
            elif word == '<prefix-list-v6>':
                value = 'JASON'
            elif word == '<route-map>':
                value = 'FOO'
            elif word == '<16-1048575>':
                value = '20'
            args[word] = value
            command.append(value)
        else:
            args[word] = True
            command.append(word)


def _expand_cmd(parsed, args=None):
    if args is None:
        args = {}
    else:
        args = deepcopy(args)

    command = []
    for idx, part in enumerate(parsed):
        if isinstance(part, dict):
            group = []
            for word in part['value']:
                if word == '|':
                    for p2, a2 in _expand_cmd(group):
                        for p3, a3 in _expand_cmd(parsed[idx + 1:], args):
                            a3.update(a2)
                            yield command + p2 + p3, a3
                    group = []
                else:
                    group.append(word)
            if len(group) > 0:
                for p2, a2 in _expand_cmd(group):
                    for p3, a3 in _expand_cmd(parsed[idx + 1:], args):
                        a3.update(a2)
                        yield command + p2 + p3, a3
            if part['op'] == 'or':
                return
        else:
            _replace_word(part, command, args)

    yield command, args


def get_possible_commands(commands):
    data = []
    for command in commands:
        _, parsed = _parse_command(command)
        for expanded, args in _expand_cmd(parsed):
            data.append((expanded, args, command))

    return data
