from nclu.tests.base import BaseTest


# then write your test cases
class TestDelAll(BaseTest):

    def test_del_all_frr_correctly(self):
        import nclu.NetDaemon  # noqa
        from nclu.plugins import frr

        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS,
            'zebra=yes\nbgp=yes')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF,
            '''frr version 4.0+cl3u6
frr defaults datacenter
hostname cel-redxp-10
!
service integrated-vtysh-config
!
password cn321
enable password cn321
!
log file /var/log/frr/zebra.log
!
log timestamp precision 6''')
        frr.del_all()
        self.assertEqual(
            self.patcher.file_expectations[frr.ETC_FRR_DAEMONS_SCRATCHPAD],
            'zebra=yes\n')
        self.assertIn(
            frr.ETC_FRR_CONF_SCRATCHPAD,
            self.patcher.file_expectations)
        pending = frr.get_pending(False)
        self.assertIn('/etc/frr/daemons', pending)
        self.assertIn('/run/nclu/frr/frr.conf', pending)

    def test_del_all_frr_does_not_detect_with_default_config(self):
        import nclu.NetDaemon  # noqa
        from nclu.plugins import frr

        # now test reset type of config that we don't
        # create a diff again
        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS,
            'zebra=yes\n')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF,
            '''frr version 4.0+cl3u3
frr defaults datacenter
hostname cumulus
username cumulus nopassword
!
service integrated-vtysh-config
!
line vty
!''')
        frr.del_all()
        self.assertNotIn(
            frr.ETC_FRR_CONF_SCRATCHPAD,
            self.patcher.file_expectations)
        pending = frr.get_pending(False)
        self.assertNotIn('/etc/frr/daemons', pending)
        self.assertNotIn('/run/nclu/frr/frr.conf', pending)

    def test_del_all_frr_does_not_detect_with_default_config_exec(self):
        import nclu.NetDaemon  # noqa
        from nclu.plugins import frr

        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS,
            'zebra=yes\n')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF,
            '''frr version 4.0+cl3u6
frr defaults datacenter
hostname cel-redxp-10
!
service integrated-vtysh-config
!
password cn321
!
line vty
 exec-timeout 0 0
!
''')
        frr.del_all()
        self.assertNotIn(
            frr.ETC_FRR_CONF_SCRATCHPAD,
            self.patcher.file_expectations)
        pending = frr.get_pending(False)
        self.assertNotIn('/etc/frr/daemons', pending)
        self.assertNotIn('/run/nclu/frr/frr.conf', pending)

    def test_del_all_frr_commit_copies_new_default(self):
        import nclu.NetDaemon  # noqa
        from nclu.plugins import frr
        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS,
            'zebra=yes\nbgp=yes')
        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS_BACKUP,
            'zebra=yes')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF,
            '''frr version 4.0+cl3u6
frr defaults datacenter
hostname cel-redxp-10
!
service integrated-vtysh-config
!
password cn321
enable password cn321
!
log file /var/log/frr/zebra.log
!
log timestamp precision 6''')
        frr.del_all()
        self.assertEqual(
            self.patcher.file_expectations[frr.ETC_FRR_DAEMONS_SCRATCHPAD],
            'zebra=yes\n')
        self.assertIn(
            frr.ETC_FRR_CONF_SCRATCHPAD,
            self.patcher.file_expectations)

        self.patcher.expect_command(
            '/bin/systemctl is-enabled frr.service', '')
        self.patcher.expect_command(
            '/bin/systemctl is-active frr.service', '')
        self.patcher.expect_command(
            '/bin/systemctl enable frr.service', '')
        self.patcher.expect_command(
            '/bin/systemctl reset-failed frr.service', '')
        self.patcher.expect_command(
            '/bin/systemctl restart frr.service', '')
        self.patcher.expect_command(
            '/usr/lib/frr/frr-reload.py --overwrite '
            '--stdout --reload /etc/frr/frr.conf', '')
        self.patcher.expect_command(
            '/usr/bin/vtysh -c wr mem', '')

        frr.commit_pending(False, True)
        self.assertIn(frr.ETC_FRR_CONF_DEFAULT,
                      self.patcher.file_expectations)

    def test_del_all_frr_uses_saved_default(self):
        import nclu.NetDaemon  # noqa
        from nclu.plugins import frr
        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS,
            'zebra=yes\nbgp=yes')
        self.patcher.expect_file(
            frr.ETC_FRR_DAEMONS_BACKUP,
            'zebra=yes')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF,
            '''frr version 4.0+cl3u6
frr defaults datacenter
hostname cel-redxp-10
!
log timestamp precision 6''')
        self.patcher.expect_file(
            frr.ETC_FRR_CONF_DEFAULT,
            'foobar saved default')

        frr.del_all()
        self.assertEqual(
            self.patcher.file_expectations[frr.ETC_FRR_CONF_SCRATCHPAD],
            'foobar saved default')
